@@ -1,5 +1,4 @@
 #! /usr/bin/perl
-# $Id: ldifdiff.pl,v 3.7 2005/03/15 14:22:45 subbarao Exp $
 
 =head1 NAME
 
@@ -7,13 +6,13 @@ ldifdiff.pl -- Generates LDIF change diff between two sorted LDIF files.
 
 =head1 DESCRIPTION
 
-ldifdiff.pl takes as input two sorted LDIF files, source and target, and 
-generates on standard output the LDIF changes needed to transform the target 
+ldifdiff.pl takes as input two sorted LDIF files, source and target, and
+generates on standard output the LDIF changes needed to transform the target
 into the source.
 
 =head1 SYNOPSIS
 
-ldifdiff.pl B<-k|--keyattr keyattr> [B<-a|--sourceattrs attr1,attr2,...>] [B<-c|--ciscmp attr1,...>] [B<--dnattrs attr1,...>] [B<--sharedattrs attr1,...>] B<sourcefile> B<targetfile>
+ldifdiff.pl B<-k|--keyattr keyattr> [B<-a|--sourceattrs attr1,attr2,...>] [B<-c|--ciscmp attr1,...>] [B<-n|--numcmp attr1,...>] [B<--dnattrs attr1,...>] [B<--sharedattrs attr1,...>] B<sourcefile> B<targetfile>
 
 =head1 OPTIONS
 
@@ -21,9 +20,9 @@ ldifdiff.pl B<-k|--keyattr keyattr> [B<-a|--sourceattrs attr1,attr2,...>] [B<-c|
 
 =item B<-k|--keyattr> keyattr
 
-Specifies the key attribute to use when comparing source and target entries. 
-Entries in both LDIF files must be sorted by this attribute for comparisons to 
-be meaningful. F<ldifsort.pl> can be used to sort LDIF files by a given 
+Specifies the key attribute to use when comparing source and target entries.
+Entries in both LDIF files must be sorted by this attribute for comparisons to
+be meaningful. F<ldifsort.pl> can be used to sort LDIF files by a given
 attribute.
 
 =item B<-a|--sourceattrs attr1,attr2,...>
@@ -33,9 +32,14 @@ source and target entries. By default, all attributes are considered.
 
 =item B<-c|--ciscmp attr1,...>
 
-(Optional) Compare values of the specified attributes case-insensitively. The 
+(Optional) Compare values of the specified attributes case-insensitively. The
 default set is: mail manager member objectclass owner uid uniqueMember
 
+=item B<-n|--numcmp attr1,...>
+
+(Optional) Compare values of the specified attributes numerically. The
+default set is: employeeNumber
+
 =item B<--dnattrs attr1,...>
 
 (Optional) Specifies a list of attributes to be treated as DNs when being
@@ -45,7 +49,7 @@ compared. The default set is: manager member owner uniqueMember
 
 (Optional) Specifies a list of attribues to be treated as "shared" attributes,
 where the source may not be a sole authoritative source. When modifying
-these attributes, separate "delete" and "add" LDIF changes are generated, 
+these attributes, separate "delete" and "add" LDIF changes are generated,
 instead of a single "replace" change. The default set is objectClass.
 
 =item B<sourcefile>
@@ -68,18 +72,23 @@ use Getopt::Long;
 use strict;
 
 my @sourceattrs;
-my (%ciscmp, %dnattrs, %sharedattrs);
+my (%ciscmp, %numcmp, %dnattrs, %sharedattrs);
 my $keyattr;
 GetOptions('a|sourceattrs=s' => sub { @sourceattrs = split(/,/, $_[1]) },
 	'c|ciscmp=s' => sub { my @a = split(/,/,lc $_[1]); @ciscmp{@a} = (1) x @a },
 	'dnattrs=s' => sub { my @a = split(/,/,lc $_[1]); @dnattrs{@a} = (1) x @a },
 	'k|keyattr=s' => \$keyattr,
+	'n|numcmp=s' => sub { my @a = split(/,/,lc $_[1]); @numcmp{@a} = (1) x @a },
 	'sharedattrs=s' => sub {my @a=split(/,/,lc $_[1]);@sharedattrs{@a}=(1) x @a}
 	);
 unless (keys %ciscmp) {
-	foreach (qw(mail manager member objectclass owner uid uniquemember)) 
+	foreach (qw(cn mail manager member o ou objectclass owner uid uniquemember))
 	{ $ciscmp{$_} = 1 }
 }
+unless (keys %numcmp) {
+	foreach (qw(employeenumber))
+	{ $numcmp{$_} = 1 }
+}
 unless (keys %dnattrs) {
 	foreach (qw(manager member owner uniquemember))
 	{ $dnattrs{$_} = 1 }
@@ -120,36 +129,37 @@ sub dnsuperior { my $rv = ($_[0] =~ /^.*?(?<!\\),(.*)/)[0]; $rv }
 sub cmpDNs
 {
 	my ($adn, $bdn) = @_;
-	my $cadn = canonical_dn($adn, casefold => 'lower'); 
+	my $cadn = canonical_dn($adn, casefold => 'lower');
 	my $cbdn = canonical_dn($bdn, casefold => 'lower');
-	if ($ciscmp{lc rdnattr($cadn)}) { $cadn = lc($cadn), $cbdn = lc($cbdn) }
+	my $rdnattr = lc rdnattr($cadn);
+	if ($ciscmp{$rdnattr}) { $cadn = lc($cadn), $cbdn = lc($cbdn) }
 
-	$cadn cmp $cbdn;
+	return $numcmp{$rdnattr} ? $cadn <=> $cbdn : $cadn cmp $cbdn;
 }
 
 sub cmpEntries
-{ 
+{
 	my ($a, $b) = @_;
 	my $dncmp = cmpDNs($a->dn, $b->dn);
 
-	if (lc($keyattr) eq 'dn') { 
+	if (lc($keyattr) eq 'dn') {
 		return ($dncmp, $dncmp);
 	}
-	else { 
+	else {
 		my $aval = $a->get_value($keyattr);
 		my $bval = $b->get_value($keyattr);
 		if ($ciscmp{$keyattr}) {
 			$aval = lc($aval);
 			$bval = lc($bval);
 		}
-		return($aval cmp $bval, $dncmp);
+		return($numcmp{$keyattr} ? $aval <=> $bval : $aval cmp $bval, $dncmp);
 	}
 }
 
 
 # Diffs two LDIF data sources
 sub diff
-{ 
+{
 	my ($source, $target) = @_;
 	my ($sourceentry, $targetentry, $incr_source, $incr_target, @ldifchanges);
 
@@ -175,15 +185,15 @@ sub diff
 
 		my ($entrycmp, $dncmp) = cmpEntries($sourceentry, $targetentry);
 
-		# Check if the current source entry has a higher sort position than 
-		# the current target. If so, we interpret this to mean that the 
+		# Check if the current source entry has a higher sort position than
+		# the current target. If so, we interpret this to mean that the
 		# target entry no longer exists on the source. Issue a delete to LDAP.
 		if ($entrycmp > 0) {
 			$targetentry->delete;
 			$ldifout->write_entry($targetentry);
             $incr_target = 1, next;
 		}
-		# Check if the current source entry has a lower sort position than 
+		# Check if the current source entry has a lower sort position than
 		# the current target entry. If so, we interpret this to mean that the
 		# source entry doesn't exist on the target. Issue an add to LDAP.
 		elsif ($entrycmp < 0) {
@@ -200,7 +210,7 @@ sub diff
 			my $rdnval = rdnval($sourceentry->dn);
 			my $newsuperior = dnsuperior($sourceentry->dn);
 			my $oldsuperior = dnsuperior($targetentry->dn);
-			my $changetype; 
+			my $changetype;
 
 			if (cmpDNs($oldsuperior, $newsuperior)) {
 				$changetype = 'moddn';
@@ -215,7 +225,7 @@ sub diff
 			$targetentry->delete('deleteoldrdn');
 			$targetentry->delete('newsuperior') if $changetype eq 'moddn';
 			delete($targetentry->{changetype});
-			
+
 			$targetentry->dn($sourceentry->dn);
 			$targetentry->replace($rdnattr, $sourceentry->get_value($rdnattr))
 				if $sourceentry->exists($rdnattr);
@@ -247,8 +257,8 @@ sub updateFromEntry
 		# add all source entry attributes
 		@attrs = $source->attributes;
 		# add any other attributes we haven't seen from the target entry
-		foreach my $tattr ($target->attributes) { 
-			push(@attrs, $tattr) unless grep(/^$tattr$/i, @attrs) 
+		foreach my $tattr ($target->attributes) {
+			push(@attrs, $tattr) unless grep(/^$tattr$/i, @attrs)
 		}
 	}
 
@@ -289,8 +299,8 @@ sub updateFromEntry
 
 		# Make changes as appropriate
 		if ($sharedattrs{$lcattr}) {
-			# For 'shared' attributes (e.g. objectclass) where $source may not 
-			# be a sole authoritative source, we issue separate delete and 
+			# For 'shared' attributes (e.g. objectclass) where $source may not
+			# be a sole authoritative source, we issue separate delete and
 			# add modifications instead of a single replace.
 			$target->delete($attr => [ values(%targetuniqvals) ])
 				if keys(%targetuniqvals);
@@ -309,8 +319,6 @@ sub updateFromEntry
 }
 
 
-=back
-
 =head1 AUTHOR
 
 Kartik Subbarao E<lt>subbarao@computer.orgE<gt>
