@@ -4,12 +4,13 @@
 # same terms as Perl itself.
 
 package Net::LDAPS;
-@Net::LDAPS::ISA = ( 'Net::LDAP' );
-$Net::LDAPS::VERSION = "0.05";
 
 use strict;
 use Net::LDAP;
 
+our @ISA = qw(Net::LDAP);
+our $VERSION = '0.06';
+
 sub new {
   shift->SUPER::new(@_, scheme => 'ldaps');
 }
